import os.path
import sys

import click
from colorama import Fore, Style

from kaz import item
from kaz.constants import kaz_home, VERSION
from kaz.util import echo_error

@click.group(invoke_without_command=True)
@click.help_option('-h', '--help')
@click.version_option(VERSION, '-v', '--version', message='%(prog)s v%(version)s')
@click.pass_context
def cli(ctx):
    """Simple local storage cli"""

    if ctx.invoked_subcommand is None:
        ctx.invoke(ls)
    pass

@cli.command(name='list')
@click.help_option('-h', '--help')
def ls():
    """Show all held items."""

    items = item.load()
    if items:
        click.echo('\n'.join(['{}{}'.format(name, Style.DIM + ' (binary)' + Style.NORMAL if type(value) is bytes else '') for name, value in items.items()]))
    else:
        click.echo(Style.DIM + 'No items' + Style.NORMAL)

def autocomplete_name(ctx, args, incomplete):
    return [name for name in item.load().keys() if name.startswith(incomplete)]

@cli.command()
@click.help_option('-h', '--help')
@click.argument('name', autocompletion=autocomplete_name)
def get(name):
    """Print the value of an item."""

    items = item.load()
    if not name in items:
        echo_error('No such item: {}'.format(name))
        return

    click.echo(items[name])

@cli.command()
@click.help_option('-h', '--help')
@click.option('--edit/--no-edit', '-e/-E', default=False, help='Open the item in the default text editor')
@click.argument('name', autocompletion=autocomplete_name)
@click.argument('value', required=False)
def set(name, edit, value):
    """Store a value in an item."""

    items = item.load()
    original_items = dict(items)
    old = items[name] if name in items else None

    if edit:
        if value is not None:
            echo_error('Value cannot be present while --editor is set.')
            return
        edited = click.edit(text=old)
        # click.edit() returns None when no changes were made, so account for that
        value = edited.rstrip() if edited is not None else old
    else:
        if value is None:
            click.echo('Value: ', nl=False)
            value = sys.stdin.buffer.read()

    items[name] = value
    item.save(items, original_items)
    if old is None:
        click.echo(Fore.GREEN + "Added '{}'".format(name) + Fore.RESET)
    else:
        if value == old:
            click.echo(Style.DIM + 'Nothing changed' + Style.NORMAL)
        else:
            click.echo(Fore.YELLOW + "Updated '{}'".format(name) + Fore.RESET)

@cli.command()
@click.help_option('-h', '--help')
@click.argument('name', autocompletion=autocomplete_name)
def remove(name):
    """Remove an item."""

    items = item.load()
    original_items = dict(items)
    if not name in items:
        echo_error('No such item: {}'.format(name))
        return

    del items[name]
    item.save(items, original_items)
    click.echo(Fore.MAGENTA + "Removed '{}'".format(name) + Fore.RESET)

@cli.command()
@click.help_option('-h', '--help')
def clear():
    """Remove all items."""

    original_items = item.load()
    count = len(original_items)
    count = len(item.load())
    item.save({}, original_items)
    click.echo((Style.DIM if count == 0 else Fore.MAGENTA) + 'Removed {} item{}'.format(count, '' if count == 1 else 's') + Style.RESET_ALL)

if __name__ == '__main__':
    cli()



from os import mkdir, remove
from os.path import join, exists
import json

from kaz.constants import kaz_home

def load():
    if not exists(kaz_home):
        return {}
    with open(join(kaz_home, 'items.json'), 'r') as f:
        items = json.load(f)
        for name, value in items.items():
            if value is None:
                # null value means it's a pointer to a binary file, load from blobs/{name}
                with open(join(kaz_home, 'blobs', name), 'rb') as blob:
                    items[name] = blob.read()
        return items

def save(items, original_items):
    if not exists(kaz_home):
        mkdir(kaz_home)

    with open(join(kaz_home, 'items.json'), 'w') as f:
        # write new item list
        for name, value in items.items():
            if type(value) is bytes:
                blobs = join(kaz_home, 'blobs')
                if not exists(blobs):
                    mkdir(blobs)
                with open(join(blobs, name), 'w+b') as blob:
                    blob.write(value)
                items[name] = None
        # check for deleted items
        for name, value in original_items.items():
            if not name in items:
                # it was deleted, now delete the blob if the item was binary
                if type(value) is bytes:
                    remove(join(kaz_home, 'blobs', name))

        json.dump(items, f)



from os.path import abspath, dirname, join

from setuptools import setup, find_packages

this_directory = abspath(dirname(__file__))
with open(join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='kaz',
    version='0.3.0.post1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['click', 'colorama'],
    entry_points='''
        [console_scripts]
        kaz=kaz.main:cli
    ''',
    # extra metadata
    description='A simple storage cli',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='storage cli',
    url='https://github.com/clabe45/kaz',
    author='Caleb Sacks',
    project_urls={
        'Bug Tracker': 'https://github.com/clabe45/kaz/issues'
    }
)
